package tk.antoine.roux.domain.model;

public final class Either<E extends Exception, T> {
    private final E error;
    private final T success;

    private Either(E error, T success) {
        this.error = error;
        this.success = success;
    }

    public Exception left() {
        return error;
    }

    public T right() {
        return success;
    }

    public static <E extends Exception, T> Either<E, T> right(T success) {
        return new Either<>(null, success);
    }

    public static <E extends Exception, T> Either<E, T> left(E error) {
        return new Either<>(error, null);
    }

    public boolean isLeft() {
        return success == null;
    }

    public boolean isRight() {
        return error == null;
    }

}
