package tk.antoine.roux.architecture;

import com.tngtech.archunit.core.domain.JavaClasses;
import com.tngtech.archunit.core.importer.ClassFileImporter;
import com.tngtech.archunit.lang.CompositeArchRule;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;

import static com.tngtech.archunit.core.importer.ImportOption.Predefined.DO_NOT_INCLUDE_ARCHIVES;
import static com.tngtech.archunit.core.importer.ImportOption.Predefined.DO_NOT_INCLUDE_JARS;
import static com.tngtech.archunit.core.importer.ImportOption.Predefined.DO_NOT_INCLUDE_PACKAGE_INFOS;
import static com.tngtech.archunit.core.importer.ImportOption.Predefined.DO_NOT_INCLUDE_TESTS;
import static com.tngtech.archunit.library.GeneralCodingRules.NO_CLASSES_SHOULD_ACCESS_STANDARD_STREAMS;
import static com.tngtech.archunit.library.GeneralCodingRules.NO_CLASSES_SHOULD_USE_FIELD_INJECTION;
import static com.tngtech.archunit.library.GeneralCodingRules.NO_CLASSES_SHOULD_USE_JAVA_UTIL_LOGGING;
import static com.tngtech.archunit.library.GeneralCodingRules.NO_CLASSES_SHOULD_USE_JODATIME;

/**
 * Tests of good practice coding usage
 */
class CodingGoodPracticeTest {
    private static final String BASE_PACKAGE = "tk.antoine.roux";

    ClassFileImporter classFileImporter = new ClassFileImporter()
        .withImportOption(DO_NOT_INCLUDE_ARCHIVES)
        .withImportOption(DO_NOT_INCLUDE_JARS)
        .withImportOption(DO_NOT_INCLUDE_PACKAGE_INFOS);

    JavaClasses classes = classFileImporter
        .importPackages(BASE_PACKAGE);

    JavaClasses classesWithoutTest = classFileImporter
        .withImportOption(DO_NOT_INCLUDE_TESTS)
        .importPackages(BASE_PACKAGE);

    @Test
    @DisplayName("Should prevent field injection everywhere, except in test")
    void preventFieldInjection() {
        CompositeArchRule commonUsageRules = CompositeArchRule
            .of(NO_CLASSES_SHOULD_USE_FIELD_INJECTION);
        commonUsageRules.check(classesWithoutTest);
    }

    @Test
    @DisplayName("Should prevent usage of system stream like out, err, in")
    void preventSystemStream() {
        CompositeArchRule commonUsageRules = CompositeArchRule
            .of(NO_CLASSES_SHOULD_ACCESS_STANDARD_STREAMS);
        commonUsageRules.check(classes);
    }

    @Test
    @DisplayName("Should prevent usage of Java.util.logging classes")
    void preventJavaUtilLogging() {
        CompositeArchRule commonUsageRules = CompositeArchRule
            .of(NO_CLASSES_SHOULD_USE_JAVA_UTIL_LOGGING);
        commonUsageRules.check(classes);
    }

    @Test
    @DisplayName("Should prevent usage of JodaTime library")
    void preventJodaTime() {
        CompositeArchRule commonUsageRules = CompositeArchRule
            .of(NO_CLASSES_SHOULD_USE_JODATIME);
        commonUsageRules.check(classes);
    }
}
